@@ -44,11 +44,12 @@
 #include <bonobo/bonobo-property-bag.h>
 #include <bonobo/bonobo-zoomable.h>
 #include <bonobo/bonobo-ui-util.h>
+#include <eel/eel-vfs-extensions.h>
 
 #include "eog-wrap-list.h"
 #include "eog-scroll-view.h"
 #include "eog-collection-view.h"
-#include "eog-collection-model.h"
+#include "eog-image-list.h"
 #include "eog-collection-marshal.h"
 #include "eog-full-screen.h"
 #include "eog-info-view.h"
@@ -83,7 +84,7 @@ static const gchar *pref_key[] = {
 };
 
 struct _EogCollectionViewPrivate {
-	EogCollectionModel      *model;
+	EogImageList            *model;
 
 	GtkWidget               *wraplist;
 	GtkWidget               *scroll_view;
@@ -113,11 +114,52 @@ enum {
 	LAST_SIGNAL
 };
 
+static void handle_selection_changed (EogWrapList *list, EogCollectionView *view);
+
 static guint eog_collection_view_signals [LAST_SIGNAL];
 
 BONOBO_CLASS_BOILERPLATE (EogCollectionView, eog_collection_view,
 			  BonoboPersistFile, BONOBO_TYPE_PERSIST_FILE);
 
+static void
+free_image_list (GList *list)
+{
+	GList *it;
+
+	for (it = list; it != NULL; it = it->next) {
+		g_object_unref (G_OBJECT (it->data));
+	}
+	g_list_free (list);
+}
+
+static void
+slideshow_hide_cb (GtkWidget *widget, gpointer data)
+{
+	EogImage *last_image;
+	EogFullScreen *fs;
+	EogCollectionView *view;
+	EogCollectionViewPrivate *priv;
+
+	fs = EOG_FULL_SCREEN (widget);
+	view = EOG_COLLECTION_VIEW (data);
+	priv = view->priv;
+
+	last_image = eog_full_screen_get_last_image (fs);
+	
+	if (last_image != NULL) {
+		g_signal_handlers_block_by_func(G_OBJECT (priv->wraplist),
+						G_CALLBACK (handle_selection_changed), view);
+
+		eog_wrap_list_set_current_image (EOG_WRAP_LIST (priv->wraplist), last_image, FALSE);
+		eog_scroll_view_set_image (EOG_SCROLL_VIEW (priv->scroll_view), last_image);
+
+		g_signal_handlers_unblock_by_func(G_OBJECT (priv->wraplist),
+						  G_CALLBACK (handle_selection_changed), view);
+		g_object_unref (last_image);
+	}
+
+	gtk_widget_destroy (widget);
+}
 
 static void
 verb_SlideShow_cb (BonoboUIComponent *uic, 
@@ -126,26 +168,29 @@ verb_SlideShow_cb (BonoboUIComponent *uic,
 {
 	EogCollectionView *view;
 	GtkWidget *show;
-	GList *images = NULL;
 	EogImage *start_image = NULL;
-	gboolean free_list = FALSE;
+	EogImageList *list;
 
 	view = EOG_COLLECTION_VIEW (user_data);
 
 	if (eog_wrap_list_get_n_selected (EOG_WRAP_LIST (view->priv->wraplist)) > 1) {
-		images = eog_wrap_list_get_selected_images (EOG_WRAP_LIST (view->priv->wraplist));
-		free_list = TRUE;
+		GList *image_list;
+
+		image_list = eog_wrap_list_get_selected_images (EOG_WRAP_LIST (view->priv->wraplist));
+		list = eog_image_list_new_from_glist (image_list);
+
+		free_image_list (image_list);
 	}
 	else {
-		images = eog_collection_model_get_image_list (view->priv->model);
+		list = g_object_ref (view->priv->model);
 	}
 	start_image = eog_wrap_list_get_first_selected_image (EOG_WRAP_LIST (view->priv->wraplist));
 
-	show = eog_full_screen_new (images, start_image);
-	g_object_unref (start_image);
+	show = eog_full_screen_new (list, start_image);
+	g_signal_connect (G_OBJECT (show), "hide", G_CALLBACK (slideshow_hide_cb), view);
 
-	if (free_list)
-		g_list_free (images);
+	g_object_unref (start_image);
+	g_object_unref (list);
 
 	gtk_widget_show (show);
 }
@@ -189,6 +234,7 @@ verb_Undo_cb (BonoboUIComponent *uic, gpointer data, const char *name)
 		EogImage *image = EOG_IMAGE (it->data);
 		eog_image_undo (image);
 	}
+	free_image_list (images);
 }
 
 /*
@@ -261,15 +307,15 @@ save_image_loading_finished (EogImage *image, gpointer data)
 	g_signal_handler_disconnect (G_OBJECT (image), ctx->failed_id);
 
 	if (eog_image_is_modified (image) && !ctx->cancel) {
-			GError *error = NULL;
-			GnomeVFSURI *uri;
-			
-			uri = eog_image_get_uri (image);
-			
-			if (!eog_image_save (image, uri, &error)) {
-				/* FIXME: indicate failed state to the user */
-			}
-			gnome_vfs_uri_unref (uri);
+		GError *error = NULL;
+		GnomeVFSURI *uri;
+		
+		uri = eog_image_get_uri (image);
+		
+		if (!eog_image_save (image, uri, NULL, &error)) {
+			/* FIXME: indicate failed state to the user */
+		}
+		gnome_vfs_uri_unref (uri);
 	}
 
 	if (image != ctx->view->priv->displayed_image) {
@@ -305,8 +351,6 @@ save_image_init_loading (SaveContext *ctx)
 		ctx->failed_id = g_signal_connect (image, "loading_failed", G_CALLBACK (save_image_loading_failed), ctx);
 		ctx->image_list = g_list_delete_link (ctx->image_list, ctx->image_list);
 
-		g_object_ref (image);
-
 		eog_image_load (image, EOG_IMAGE_LOAD_COMPLETE);
 	}
 
@@ -349,7 +393,7 @@ save_dlg_response_cb (GtkDialog *dialog,
 	case GTK_RESPONSE_OK:
 		view = ctx->view;
 		if (ctx->image_list != NULL) {
-			g_list_free (ctx->image_list);
+			free_image_list (ctx->image_list);
 		}
 		g_free (ctx);
 		g_timeout_add (1250, save_dlg_close, dialog);
@@ -470,7 +514,8 @@ apply_transformation (EogCollectionView *view, EogTransform *trans)
 		g_signal_handler_disconnect (image, id);
 		data->counter++;
 	}
-
+	
+	free_image_list (images);
 	g_free (data);
 	g_object_unref (trans);
 }
@@ -606,6 +651,12 @@ eog_collection_view_create_ui (EogCollectionView *view)
 						     collection_verbs,
 						     view);
 
+	/* special case for SUN keyboards where F11 key has different keysym */
+	if (eog_full_screen_enable_SunF36()) {
+		bonobo_ui_component_add_verb (priv->uic, "FullScreenHiddenSun",
+					      verb_SlideShow_cb, view);
+	}
+
 	if (!priv->has_zoomable_frame) {
 		bonobo_ui_util_set_ui (priv->uic, DATADIR, "eog-image-view-ctrl-ui.xml", "EogCollectionView", NULL);
 		
@@ -796,15 +847,13 @@ eog_collection_view_get_prop (BonoboPropertyBag *bag,
 		gchar *base_uri;
 		gchar *title;
 
-		base_uri = eog_collection_model_get_base_uri (priv->model);
+		/* FIXME: what should happen with this? */
+		base_uri = NULL; /*  eog_collection_model_get_base_uri (priv->model); */
 
 		if (base_uri == NULL)
 			title = g_strdup (_("Collection View"));
 		else {
-			if (g_strncasecmp ("file:", base_uri, 5) == 0)
-				title = g_strdup ((base_uri+5*sizeof(guchar)));
-			else
-				title = g_strdup (base_uri);
+			title = eel_format_uri_for_display (base_uri);
 		}
 
 		BONOBO_ARG_SET_STRING (arg, title);
@@ -815,7 +864,7 @@ eog_collection_view_get_prop (BonoboPropertyBag *bag,
 		gchar *str;
 		gint nimg, nsel;
 			
-		nimg = eog_collection_model_get_length (priv->model);
+		nimg = eog_image_list_length (priv->model);
 		nsel = eog_wrap_list_get_n_selected (EOG_WRAP_LIST (priv->wraplist));
 
 		str = g_new0 (guchar, 70);
@@ -916,7 +965,6 @@ handle_selection_changed (EogWrapList *list, EogCollectionView *view)
 		if (priv->displayed_image != NULL) {
 			g_signal_handler_disconnect (priv->displayed_image, priv->progress_handler_id);
 
-			eog_image_free_mem (priv->displayed_image);
 			g_object_unref (priv->displayed_image);
 			priv->displayed_image = NULL;
 		}
@@ -934,7 +982,7 @@ handle_selection_changed (EogWrapList *list, EogCollectionView *view)
 }
 
 static void
-model_size_changed (EogCollectionModel *model, GList *id_list,  gpointer data)
+model_size_changed (EogImageList *model, GList *id_list,  gpointer data)
 {
 	EogCollectionView *view;
 
@@ -947,7 +995,7 @@ model_size_changed (EogCollectionModel *model, GList *id_list,  gpointer data)
 
 
 static void
-model_base_uri_changed (EogCollectionModel *model, gpointer data)
+model_base_uri_changed (EogImageList *model, gpointer data)
 {
 	EogCollectionView *view;
 
@@ -959,23 +1007,6 @@ model_base_uri_changed (EogCollectionModel *model, gpointer data)
 	update_title_text (view);	
 }
 
-static void
-layout_changed_cb (GConfClient *client, guint cnxn_id, 
-		   GConfEntry *entry, gpointer user_data)
-{
-	EogCollectionView *view;
-	gint layout;
-
-	g_return_if_fail (user_data != NULL);
-	g_return_if_fail (EOG_IS_COLLECTION_VIEW (user_data));
-
-	view = EOG_COLLECTION_VIEW (user_data);
-
-	layout = gconf_value_get_int (entry->value);
-	eog_wrap_list_set_layout_mode (EOG_WRAP_LIST (view->priv->wraplist), 
-				      layout);
-}
-
 static void
 zoomable_set_frame_cb (BonoboZoomable *zoomable, EogCollectionView *view)
 {
@@ -1091,7 +1122,7 @@ load_uri_cb (BonoboPersistFile *pf, const CORBA_char *text_uri,
 	if (text_uri == CORBA_OBJECT_NIL) return 0;
 
 	if (priv->model == NULL) {
-		priv->model = eog_collection_model_new ();
+		priv->model = eog_image_list_new ();
 		eog_wrap_list_set_model (EOG_WRAP_LIST (priv->wraplist), priv->model);
 
 		/* construct widget */
@@ -1112,7 +1143,7 @@ load_uri_cb (BonoboPersistFile *pf, const CORBA_char *text_uri,
 
 	}
 
-	eog_collection_model_add_uri (priv->model, (gchar*)text_uri); 
+	eog_image_list_add_directory (priv->model, (gchar*)text_uri); 
 
 	return 0;
 }
@@ -1121,13 +1152,20 @@ static GtkWidget*
 create_user_interface (EogCollectionView *list_view)
 {
 	EogCollectionViewPrivate *priv;
+#if HAVE_EXIF
 	GtkWidget *hpaned;
+#endif
 	GtkWidget *vpaned;
 	GtkWidget *sw;
 	GtkWidget *frame;
 	
 	priv = list_view->priv;
 
+	/* the upper part contains the image view, the
+	 * lower part contains the thumbnail list
+	 */
+	vpaned = eog_vertical_splitter_new ();
+
 	/* the image view for the full size image */
  	priv->scroll_view = eog_scroll_view_new ();
 	g_object_set (G_OBJECT (priv->scroll_view), "height_request", 250, NULL);
@@ -1139,23 +1177,34 @@ create_user_interface (EogCollectionView *list_view)
 	gtk_container_add (GTK_CONTAINER (frame), priv->scroll_view);
 
 #if HAVE_EXIF
-	/* info view for additional image information */
+	/* If we have additional information through libexif, we 
+	 * create an info view widget and put it to left of the 
+	 * image view. Using an eog_horizontal_splitter for this. 
+	 */
 	priv->info_view = gtk_widget_new (EOG_TYPE_INFO_VIEW, NULL);
 	sw = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), 
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
 	gtk_container_add (GTK_CONTAINER (sw), priv->info_view);
-#else
-	priv->info_view = NULL;
-	sw = NULL;
-#endif
 
-	hpaned = eog_horizontal_splitter_new ();
+	/* left side holds the image view, right side the info view */
+	hpaned = eog_horizontal_splitter_new (); 
 	gtk_paned_pack1 (GTK_PANED (hpaned), frame, TRUE, TRUE);
 	gtk_paned_pack2 (GTK_PANED (hpaned), sw, FALSE, TRUE);
 	gtk_widget_show_all (hpaned);
 
+	gtk_paned_pack1 (GTK_PANED (vpaned), hpaned, TRUE, TRUE);
+
+#else
+	/* If libexif isn't available we put the image view (frame)
+	 * directly into the vpaned created above.
+	 */
+	priv->info_view = NULL;
+	gtk_widget_show_all (frame);
+	gtk_paned_pack1 (GTK_PANED (vpaned), frame, TRUE, TRUE);	
+#endif
+
 	/* the wrap list for all the thumbnails */
 	priv->wraplist = eog_wrap_list_new ();
 	g_object_set (G_OBJECT (priv->wraplist), 
@@ -1174,11 +1223,7 @@ create_user_interface (EogCollectionView *list_view)
 					GTK_POLICY_AUTOMATIC);
 	gtk_container_add (GTK_CONTAINER (sw), priv->wraplist);
 	gtk_widget_show_all (sw);
-
-	/* put it all together */
-	vpaned = eog_vertical_splitter_new ();
 	
-	gtk_paned_pack1 (GTK_PANED (vpaned), hpaned, TRUE, TRUE);
 	gtk_paned_pack2 (GTK_PANED (vpaned), sw, TRUE, TRUE);
 
 	/* by default make the wrap list keyboard active */
